@@ -4,7 +4,7 @@ use 5.006;
 use strict;
 use warnings;
 use vars qw( @EXPORT_OK %EXPORT_TAGS
-    @bad_stashes $TRACE_FH $file $line $sub );
+    @bad_stashes $TRACE_FH $file $line $sub $trace_removed );
 
 use subs (
     qw( all_starts all_roots anon_sub recalc_sub_cache ),
@@ -25,11 +25,11 @@ B::Utils1 - Helper functions for op tree manipulation
 
 =head1 VERSION
 
-1.02
+1.03
 
 =cut
 
-our $VERSION = '1.02';
+our $VERSION = '1.03';
 
 use base 'DynaLoader';
 bootstrap B::Utils1 $VERSION;
@@ -87,7 +87,7 @@ sub _FALSE () { !!0 }
 # =item C<$op-E<gt>other>
 #
 # Normally if you call first, last or other on anything which is not an
-# UNOP, BINOP or LOGOP respectivly it will die.  This leads to lots of
+# UNOP, BINOP or LOGOP respectively it will die.  This leads to lots of
 # code like:
 #
 #     $op->first if $op->can('first');
@@ -104,7 +104,7 @@ sub _FALSE () { !!0 }
 =item C<$op-E<gt>oldname>
 
 Returns the name of the op, even if it is currently optimized to null.
-This helps you understand the stucture of the op tree.
+This helps you understand the structure of the op tree.
 
 =cut
 
@@ -372,7 +372,7 @@ it to C<0> will limit the dump to the current op.
 
 C<attributes> is a list of attributes to include in the produced
 pattern. The attributes that can be checked against in this way
-are 
+are:
 
   name targ type seq flags private pmflags pmpermflags.
 
@@ -384,7 +384,7 @@ sub B::OP::as_opgrep_pattern {
 
   my $attribs = $opt->{attributes};
   $attribs ||= [qw(name flags)];
-  
+
   my $pattern = {};
   foreach my $attr (@$attribs) {
     $pattern->{$attr} = $op->$attr() if $op->can($attr);
@@ -578,6 +578,9 @@ more expected semantics.
 All the C<walk> functions set C<$B::Utils1::file>, C<$B::Utils::line>,
 and C<$B::Utils1::sub> to the appropriate values of file, line number,
 and sub name in the program being examined.
+Sets C<$B::Utils::trace_removed> when the nextstate COPs that contained
+that line was optimized away. Such lines won't normally be
+step-able or breakpoint-able in a debugger without special work.
 
 =cut
 
@@ -602,6 +605,16 @@ sub _walkoptree_simple {
     if ( ref $op and $op->isa("B::COP") ) {
         $B::Utils1::file = $op->file;
         $B::Utils1::line = $op->line;
+        $B::Utils1::trace_removed = _FALSE;
+    } elsif ( !$op->isa('B::NULL') and $op->oldname =~ /^(next|db)state$/) {
+        # COP nextstate has been optimized away.  However by turning
+        # this locally back into a COP we can retrieve the file and line
+        # values.
+        my $cop = $op;
+        bless $cop, 'B::COP';
+        $B::Utils1::file = $cop->file;
+        $B::Utils1::line = $cop->line;
+        $B::Utils1::trace_removed = _TRUE;
     }
 
     $callback->( $op, $data );
@@ -803,13 +816,13 @@ of interest. Example:
     },
     $root_op
   );
-  
+
   if ($result) {
     my $name = $result->{notreached}->name; # result is *not* the root op
     carp("Statement unlikely to be reached (op name: $name)");
     carp("\t(Maybe you meant system() when you said exec()?)\n");
   }
-  
+
 While the above is a terribly contrived example, consider the win for a
 deeply nested pattern or worse yet, a pattern with many disjunctions.
 If a C<capture> property is found anywhere in
@@ -826,7 +839,7 @@ You cannot capture disjunctions, but that doesn't really make sense anyway.
 
 Same as above, except that you don't have to chain the conditions
 yourself.  If you pass an array-ref, opgrep will chain the conditions
-for you using C<next>. 
+for you using C<next>.
 The conditions can either be strings (taken as op-names), or
 hash-refs, with the same testable conditions as given above.
 
@@ -915,7 +928,7 @@ CONDITION:
 
                         # Fail if any entries match.
                         $_ ne $val
-                            or next CONDITION 
+                            or next CONDITION
                             for @{ $condition->{$test} }
                             [ 1 .. $#{ $condition->{$test} } ];
                     }
@@ -923,8 +936,8 @@ CONDITION:
 
                         # Fail if no entries match.
                         my $okay = 0;
-                        
-                        $_ eq $val and $okay = 1, last 
+
+                        $_ eq $val and $okay = 1, last
                             for @{ $condition->{$test} };
 
                         next CONDITION if not $okay;
@@ -958,7 +971,7 @@ CONDITION:
                     $capture->{$_} = $result->{$_} foreach keys %$result;
                 }
             }
-  
+
             # Apply all kids conditions. We $op->can(kids) (see above).
             if (exists $condition->{kids}) {
                 my $kidno = 0;
@@ -972,7 +985,7 @@ CONDITION:
 
                     my ($result) = opgrep( $kidconditions->[$kidno++], $kid );
                     next CONDITION if not $result;
-                    
+
                     if (not blessed($result)) {
                         # copy over the captured data/ops from the recursion
                         $capture->{$_} = $result->{$_} foreach keys %$result;
@@ -1033,7 +1046,7 @@ Example:
     first => { name => 'rv2hv', },
     'last' => { name => 'const', },
   };
-  
+
   my @ops = opgrep( {
       name => 'leavesub',
       first => {
@@ -1080,10 +1093,10 @@ sub op_or {
 # sub op_pattern_match {
 #   my $op = shift;
 #   my $pattern = shift;
-# 
+#
 #   my $ret = {};
-# 
-#   
+#
+#
 #   return $ret;
 # }
 
